#!/usr/bin/python
# -*- coding: utf-8 -*-

#
# Copyright (C) Big-Cat Software Association, 2007.
#
# GILDE project. 
# (Graphical Interfaces to Learn Debian Easily)
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2 of the License, or (at your
# option) any later version. Please read the COPYING file.
#
# 
# Copyright (C) 2005 by S.Çağlar Onur <caglar@uludag.org.tr>
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the
# Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

''' Standart Python Modules '''
import os
import sys
import array
import fcntl
import socket
import struct


''' Import Gettext Support '''
import gettext
__trans = gettext.translation('PyWireless', fallback=True)
_  =  __trans.ugettext

''' PyQt and PyKDE Modules'''
from qt import QToolTip, QTimer, QMessageBox, SIGNAL
from kdecore import KIcon, KIconLoader, KCmdLineArgs, KAboutData, KUniqueApplication, KStandardDirs
from kdeui import KSystemTray
from dcopexport import DCOPExObj

class WirelessStatus:

    ''' Signals '''
    SIOCGIWESSID = 0x8B1B # Get ESSID
    SIOCGIWMODE = 0x8B07  # Get Mode
    SIOCGIWRATE = 0x8B21  # Get Rate
    
    ''' Wireless Constans '''
    wKILO = 10**3
    wMEGA = 10**6
    wGIGA = 10**9

    ''' Byte constants '''
    bKILO = 2**10
    bMEGA = 2**20
    bGIGA = 2**30
    
    modes = ['Auto', 'Ad-Hoc', 'Managed', 'Master', 'Repeat', 'Second', 'Monitor']

    def __init__(self, parent):
# In order to calling functions from parent:
        self.parent=parent
        ''' Constuctor '''
        self.class_path = '/sys/class/net'
        self.device = self.findWirelessInterface()
        self.sockfd = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        if not self.device:
            QMessageBox.information(None, _("No Wireless Interface"), _("You don't have any wireless interface..."), QMessageBox.Ok)
#            sys.exit(1)


# Check KWireless Tools Pro config dir:
        self.user_kwirelesstoolspro=os.path.expanduser("~/.kwirelesstoolspro")
        print self.user_kwirelesstoolspro

        if not os.path.exists(self.user_kwirelesstoolspro):
            os.mkdir(self.user_kwirelesstoolspro)



    def findWirelessInterface(self):
        ''' Finds wireless interface '''
        fileName = 'wireless'
        for interface in os.listdir(self.class_path):
            if os.path.exists(os.path.join(self.class_path, interface, fileName)):
                return interface
#            else:
#                return "Nonem"

    def returnInterfaceStatus(self):
        fileName = 'carrier'
        try:
            self.status = file(os.path.join(self.class_path, self.device, fileName)).readline().strip()
        except IOError:
            return 0
        return self.status

    def returnInterfaceName(self):
        ''' Returns the wireless interface name '''
        return self.device

    def returnLinkStatus(self):
        ''' Returns wireless link status % '''
        fileName = 'wireless/link'
        self.link = file(os.path.join(self.class_path, self.device, fileName)).readline().strip()
        return int(self.link)
    
    def returnNoiseStatus(self):
        ''' Returns current noise level '''
        fileName = 'wireless/noise'
        self.noise = file(os.path.join(self.class_path, self.device, fileName)).readline().strip()
        return int(self.noise) - 256

    def returnSignalStatus(self):
        ''' Returns current signal level '''
        fileName = 'wireless/level'
        self.signal = file(os.path.join(self.class_path, self.device, fileName)).readline().strip()
        return int(self.signal) - 256

    def returnReceived(self):
        ''' Returns received bytes '''
        fileName = 'statistics/rx_bytes'
        self.rx = int(file(os.path.join(self.class_path, self.device, fileName)).readline().strip())
 
        if self.rx >= self.bGIGA:
            return "%i Gb" %(self.rx/self.bGIGA)

        if self.rx >= self.bMEGA:
            return "%i Mb" %(self.rx/self.bMEGA)

        return "%i Kb" %(self.rx/self.bKILO)

    def returnTransferred(self):
        ''' Returns transferred bytes '''
        fileName = 'statistics/tx_bytes'
        self.tx = int(file(os.path.join(self.class_path, self.device, fileName)).readline().strip())
 
        if self.tx >= self.bGIGA:
            return "%i Gb" %(self.tx/self.bGIGA)

        if self.tx >= self.bMEGA:
            return "%i Mb" %(self.tx/self.bMEGA)

        return "%i Kb" %(self.tx/self.bKILO)

    def returnESSID(self):
        ''' Returns essid of interface '''
        buffer, structure = self.packRequest(32)
        i, result = self.readInformation(self.SIOCGIWESSID, structure)
        if i > 0:
            return result
        return buffer.tostring().strip('\x00')

    def returnBitrate(self):
        ''' Returns bit rate of interface '''
        i, result = self.readInformation(self.SIOCGIWRATE)
        if i > 0:
            return result

        size = struct.calcsize('ihbb')
        m, e, i, pad = struct.unpack('ihbb', result[:size])
        if e == 0:
            bitrate =  m
        else:
            bitrate = float(m) * 10**e

        if bitrate >= self.wGIGA:
            return "%i Gb/s" %(bitrate/self.wGIGA)

        if bitrate >= self.wMEGA:
            return "%i Mb/s" %(bitrate/self.wMEGA)

        if bitrate >= self.wKILO:
            return "%i Kb/s" %(bitrate/self.wKILO)

    def returnMode(self):
        ''' Returns operation mode of interface '''
        i, result = self.readInformation(self.SIOCGIWMODE)
        if i > 0:
            return result
        mode = self.unpackRequest('i', result[:4])[0]
        return self.modes[mode]

    ''' Internal Methods '''

    def packRequest(self, bufferSize):
        """ Packs wireless request data for sending it to the kernel """
        buffer = array.array('c', '\0' * bufferSize)
        caddr_t, length = buffer.buffer_info()
        structure = struct.pack('Pi', caddr_t, length)
        return buffer, structure

    def unpackRequest(self, format, packedRequest):
        """ Unpacks request with given format """
        return struct.unpack(format, packedRequest)

    def readInformation(self, request, data = None):
        ''' Read information from interface '''
        if data is not None:
            buffer = 16 - len(self.device)
            requestedInterface = self.device + '\0' * buffer
            requestedInterface += data
        else:
            requestedInterface = (self.device + '\0' * 32)
        try:
            result = fcntl.ioctl(self.sockfd.fileno(), request, requestedInterface)
        except IOError, (i, e):
            return i, e

        return (0, result[16:])

class DCOPIface(DCOPExObj):
    def __init__(self, wirelessStatusObject):
        ''' Constructor '''
        
        ''' Initialize DCOP ''' 
        DCOPExObj.__init__(self, "PyWirelessIface")
   
        self.wSO = wirelessStatusObject

        ''' DCOP functions '''
        self.addMethod("QString getInterfaceName()", self.getInterfaceName)
        self.addMethod("int getLinkStatus()", self.getLinkStatus)
        self.addMethod("int getNoiseStatus()", self.getNoiseStatus)
        self.addMethod("int getSignalStatus()", self.getSignalStatus)
        self.addMethod("QString getESSID()", self.getESSID)
        self.addMethod("QString getMode()", self.getMode)
        self.addMethod("QString returnReceived()", self.returnReceived)
        self.addMethod("QString returnTransferred()", self.returnTransferred)

    def getInterfaceName(self):
        return self.wSO.returnInterfaceName()

    def getLinkStatus(self):
        ''' Returns wireless link status % '''
        return self.wSO.returnLinkStatus()

    def getNoiseStatus(self):
        ''' Returns current noise level '''
        return self.wSO.returnNoiseStatus()

    def getSignalStatus(self):
        ''' Returns current signal level '''
        return self.wSO.returnSignalStatus()

    def getESSID(self):
        ''' Returns essid of interface '''
        return self.wSO.returnESSID()

    def getMode(self):
        ''' Returns operation mode of interface '''
        return self.wSO.returnMode()

    def returnReceived(self):
        ''' Returns received bytes '''
        return self.wSO.returnReceived()

    def returnTransferred(self):
        ''' Returns transferred bytes '''
        return self.wSO.returnTransferred()

class SystemTray(KSystemTray):
    def __init__(self, *args):
        apply(KSystemTray.__init__, (self,) + args)
# reference to WirelessStatus is also self.wirelessStatus=...

        ''' WirelessStatus instance '''
        self.wirelessStatus = WirelessStatus(self)

        ''' DCOP Interface ''' 
        self.DCOPInstance = DCOPIface(self.wirelessStatus)

        ''' Add /usr/share/PyWireless to KStandardDirs '''
        self.KStandardDirs =  KStandardDirs()
        self.KStandardDirs.addResourceDir("icon", "/usr/share/PyWireless/icons")
        ''' Tray icon Loader '''
        self.icons = KIconLoader("PyWireless", self.KStandardDirs)

        ''' Load icon on startup '''
        self.setPixmap(self.icons.loadIcon('pywireless', KIcon.Desktop, 22)) 

        ''' Timer event triggered every 5 second
            Until i found a way to use inotify or libfam '''
        self.time = QTimer(self)
        self.connect(self.time, SIGNAL('timeout()'), self.timeoutSlot)
        self.time.start(3000)

        self.connect(app, SIGNAL("shutDown()"), self.slotQuit)


# future plan: select interface
#        self.contextMenu().insertItem("Search Interface", self.slotSearchIF)
#        self.contextMenu().insertTitle("USAGE")

#        self.contextMenu().insertItem("Hide X", self.setInactive)


        self.contextMenu().insertItem("Manage Connections", self.slotGuifi)

        self.contextMenu().insertItem("Continuous Monitor", self.slotMonitor)

# version 2:
        self.contextMenu().insertItem("Setup Default Connection", self.slotConfigurator)
        self.contextMenu().insertItem("Start/Stop Default Connection", self.slotStartStop)

        self.contextMenu().insertTitle("HELP & ABOUT")
        self.contextMenu().insertItem("Help", self.slotHelp)


        self.contextMenu().insertItem("About KWireless Tools Pro", self.slotAboutWirelessManager)
        self.contextMenu().insertItem("About QT", self.slotAboutQT)




        self.show()


# future plan: select interface
#    def slotSearchIF(self):
#        asd=self.wirelessStatus.findWirelessInterface()
#        print asd
#        self.wirelessStatus.device=asd


    def slotHelp(self):
        os.system("khelpcenter /usr/share/doc/kde/HTML/en/kwireless-tools-pro/index.html &")

    def slotGuifi(self):
        os.system("kdesu 'perl /usr/share/PyWireless/guifi.pl' &")

    def slotMonitor(self):
        os.system("python /usr/share/PyWireless/monitorWindow.py &")

# version 2
    def slotConfigurator(self):
        os.system("kdesu 'python /usr/share/PyWireless/kwifiproperties.py' &")

    def slotStartStop(self):
        os.system("kdesu 'python /usr/share/PyWireless/ifupdown.py' &")

    def slotAboutWirelessManager(self):
        os.system("python /usr/share/PyWireless/aboutform.py &")

    def slotAboutQT(self):
        QMessageBox.aboutQt(self, "About Qt3")


    def slotQuit(self):
        self.deleteLater()
        app.quit()


    def timeoutSlot(self):
#        if self.wirelessStatus.device == "Nonem":
        self.wirelessStatus.device = self.wirelessStatus.findWirelessInterface()
        if self.wirelessStatus.device:
            interfaceName = self.wirelessStatus.returnInterfaceName()
            interfaceESSID = self.wirelessStatus.returnESSID()
            interfaceMode = self.wirelessStatus.returnMode()
            linkStatus = self.wirelessStatus.returnLinkStatus()
            noiseStatus = self.wirelessStatus.returnNoiseStatus()
            signalStatus = self.wirelessStatus.returnSignalStatus()
            #S/N ratio:
            SperNStatus = signalStatus - noiseStatus
            bitRate = self.wirelessStatus.returnBitrate()
            received = self.wirelessStatus.returnReceived()
            transferred = self.wirelessStatus.returnTransferred()
            status = self.wirelessStatus.returnInterfaceStatus()

        ''' Tray icon name '''
        if self.wirelessStatus.device and int(status):

            if linkStatus < 5:
                index = 0
            elif linkStatus < 10:
                index = 1
            elif linkStatus < 20:
                index = 2
            elif linkStatus < 40:
                index = 3
            elif linkStatus < 70:
                index = 4
            else:
                index = 5

#            index = int(linkStatus) / 20
            iconName = 'pywireless_' + str(index)

            ''' Tooltip '''
            toolTip = _('''<center><img align="center" src="/usr/share/PyWireless/icons/%s.png"></center>
            <center>
            <table border="0" bgcolor="#000000" cellspacing="1" cellpadding="1">
            <tr>
                <td colspan="2" bgcolor="#04CC1A"><center><b>Monitoring:</b> [ %s ]</b></center></td>
            <tr>
                <td bgcolor="#CCCCCC"><b>ESSID:</b></td>
                <td bgcolor="#CCCCCC"><center>%s</center></td>
            </tr>
            <tr>
                <td bgcolor="#EEEEEE"><b>Link Quality:</b></td>
                    <td bgcolor="#EEEEEE"><center>%d</center></td>
            </tr>
            <tr>
                <td bgcolor="#CCCCCC"><b>S/N Ratio:</b></td>
                <td bgcolor="#CCCCCC"><center>%d dBm</center></td>
            </tr>
            <tr>
                <td bgcolor="#EEEEEE"><b>Signal Level:</b></td>
                <td bgcolor="#EEEEEE"><center>%d dBm</center></td>
            </tr>
            <tr>
                <td bgcolor="#CCCCCC"><b>Bitrate:</b></td>
                <td bgcolor="#CCCCCC"><center>%s</center></td>
            </tr>
            <tr>
               <td bgcolor="#EEEEEE"><b>Mode:</b></td>
                <td bgcolor="#EEEEEE"><center>%s</center></td>
            </tr>
            <tr>
                <td bgcolor="#CCCCCC"><b>Received:</b></td>
                <td bgcolor="#CCCCCC"><center>%s</center></td>
            </tr>
            <tr>
                <td bgcolor="#EEEEEE"><b>Transferred:</b></td>
                <td bgcolor="#EEEEEE"><center>%s</center></td>
            </tr>
            </table>
            </center>
            ''') % (iconName, interfaceName, interfaceESSID, linkStatus, SperNStatus, signalStatus, bitRate, interfaceMode, received, transferred)
        elif self.wirelessStatus.device:
            iconName = 'pywireless'
            toolTip = _('''<center><img align="center" src="/usr/share/PyWireless/icons/%s.png"></center>
            <center>
            <table border="0" bgcolor="#000000" cellspacing="1" cellpadding="1">
            <tr>
                <td colspan="2" bgcolor="#DD0500"><center><b>Interface '%s' <br>is powered off</b></center></td>
            <tr>
            </table>
            </center>''') % (iconName, interfaceName)

        else:
            iconName = 'pywireless_none'
            toolTip = _('''<center><img align="center" src="/usr/share/PyWireless/icons/%s.png"></center>
            <center>
            <table border="0" bgcolor="#000000" cellspacing="1" cellpadding="1">
            <tr>
                <td colspan="2" bgcolor="#DD0500"><center> <b>Wireless interface<br>not found</b></center></td>
            <tr>
            </table>
            </center>''') % (iconName)


##                <td colspan="2" bgcolor="#DD0500"><center>[ %s ] <b>is powered off</b></center></td>
##            </center>''') % (iconName, interfaceName)

        QToolTip.add(self, toolTip)
        self.setPixmap(self.icons.loadIcon(iconName, KIcon.Desktop, 22))

if __name__ == '__main__':
    appName = 'PyWireless'
    programName = 'PyWireless'
    description = 'A Basic Wireless Connection Monitor'
    license = KAboutData.License_GPL
    version = '3.0'
    NameVersion = 'KWireless Tools Pro 1.0'
    copyright = '(C) 2005 S.Çağlar Onur <caglar@uludag.org.tr>'

    sys.argv.extend(['--icon', '/usr/share/PyWireless/icons/pywireless_5.png', '--caption', NameVersion])


    aboutData = KAboutData(appName, programName, version, description, license, copyright)

    aboutData.addAuthor('S.Çağlar Onur', 'Maintainer', 'caglar@uludag.org.tr')
    aboutData.addAuthor('Onur Küçük', 'Contributor [Rx/Tx bytes part]', 'onur@uludag.org.tr')
    aboutData.addAuthor('Serdar Soytetir', 'Contributor [New PyWireless Icons & UI Improvements]', 'sendirom@gmail.com')
    aboutData.addAuthor('Furkan Duman', 'Contributor [Bug fix]', 'coderlord@yahoo.com')

    KCmdLineArgs.init(sys.argv, aboutData)
    KCmdLineArgs.addCmdLineOptions([('+files', 'File to open')])

    ''' Use KUniqueApplication and initialize'''
    gettext.install(appName)
    app = KUniqueApplication(True, True, True)
    trayWindow = SystemTray(None, appName)
    app.setMainWidget(trayWindow)

    ''' Enter main loop '''
    app.exec_loop()
