#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) Big-Cat Software Association, 2007.
#
# GILDE project.
# (Graphical Interfaces to Learn Debian Easily)
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2 of the License, or (at your
# option) any later version. Please read the COPYING file.
#

# This source is from: "The Python version of qwt-*/examples/cpuplot"

import os, sys
from qt import *
from Qwt4.Qwt import *
from Qwt4.anynumpy import *
from Numeric import *

class WirelessStat:

    LinkQuality = 0

    SNRatio = 2

    axisScale = 100

    def __init__(self):

        self.class_path = '/sys/class/net'
        self.device = self.findWirelessInterface()

        self.axisScale = 100
    # __init__()

# kell-e?
    def returnInterfaceName(self):
        ''' Returns the wireless interface name '''
        return self.device


    def findWirelessInterface(self):
        ''' Finds wireless interface '''
        fileName = 'wireless'
        for interface in os.listdir(self.class_path):
            if os.path.exists(os.path.join(self.class_path, interface, fileName)):
                return interface


    def upTime(self):
        result = QTime()
        return result

    # upTime()




    def WirelessStatRead(self):
        if os.path.exists(os.path.join(self.class_path, self.device)):
            link = file(os.path.join(self.class_path, self.device, 'wireless/link')).readline().strip()

            noise = file(os.path.join(self.class_path, self.device, 'wireless/noise')).readline().strip()
#        return int(self.noise) - 256

            signal = file(os.path.join(self.class_path, self.device, 'wireless/level')).readline().strip()
# link is also for midscale
            print link
            print signal
            print noise


            return float(link), float(int(signal)-int(noise))
        else:
            return 0.0

    # WirelessStatRead()


# class WirelessStat


class TimeScaleDraw(QwtScaleDraw):

    def __init__(self, baseTime, *args):
        apply(QwtScaleDraw.__init__, (self,) + args)
        self.baseTime = baseTime
        #self.setLabelRotation(-50.0)
        #self.setLabelAlignment(Qt.AlignLeft | Qt.AlignBottom)

    # __init__()

    def label(self, value):
        upTime = self.baseTime.addSecs(int(value))
        # test only
        if upTime.toString()=="00:01:30":
            self.axisScale = 20
#        print upTime.toString()
        return upTime.toString()

    # label()

# class TimeScaleDraw


# class CpuPieMarker
                        
    
HISTORY = 60


class WirelessPlot(QwtPlot):

    def __init__(self, *args):
        apply(QwtPlot.__init__, (self,) + args)

        self.data = {}
        self.keys = {}
        self.timeData = 1.0*arange(HISTORY-1, -1, -1)
        self.WirelessStat = WirelessStat()

        self.plotLayout().setCanvasMargin(0)
        self.plotLayout().setAlignCanvasToScales(1)
        self.setCanvasBackground(Qt.white)
        #Qt.darkGray, darkBlue
        self.setAutoLegend(1)
        self.setLegendPos(Qwt.Right)
        
        self.setAxisTitle(QwtPlot.xBottom, "Time")

        self.setAxisScaleDraw(QwtPlot.xBottom,
                              TimeScaleDraw(self.WirelessStat.upTime()))
        self.setAxisScale(QwtPlot.xBottom, 0, HISTORY)
        self.setAxisLabelRotation(QwtPlot.xBottom, -50.0)
        self.setAxisLabelAlignment(QwtPlot.xBottom,
                                   Qt.AlignLeft | Qt.AlignBottom)

        self.setAxisTitle(QwtPlot.yLeft, "dBm")
# Set axis scale
        self.setAxisScale(QwtPlot.yLeft, 0, self.WirelessStat.axisScale)

        key = self.insertCurve("Link Quality")
        self.setCurvePen(key, QPen(Qt.blue))
        self.setCurveBrush(key, QBrush(QColor(), QBrush.SolidPattern))
        self.data['LinkQuality'] = zeros(HISTORY, Float)
        self.keys['LinkQuality'] = key
# QBrush.NoBrush, Dense6Pattern, Dense7Pattern
        key = self.insertCurve("S/N Ratio")
        self.setCurvePen(key, QPen(Qt.red))
        self.setCurveBrush(key, QBrush(QColor(), QBrush.Dense4Pattern))
        self.data['SNRatio'] = zeros(HISTORY, Float)
        self.keys['SNRatio'] = key


#        self.insertMarker(CpuPieMarker(self))

        self.startTimer(1000)
        self.replot()

        self.connect(self, SIGNAL("legendClicked(long)"), self.toggleCurve)

    # __init__()
    
    def drawCanvas(self, p):
        yMap = self.canvasMap(QwtPlot.yLeft)

        r = p.window()
        r.setHeight(yMap.transform(80.0) - r.top())
        p.fillRect(r, QBrush(Qt.white))

        r.setTop(yMap.transform(80.0))
        r.setHeight(yMap.transform(40.0) - yMap.transform(80.0))
        p.fillRect(r, QBrush(Qt.white))
#Qt.gray
        QwtPlot.drawCanvas(self, p)

    # drawCanvas()
    
    def timerEvent(self, e):
        for key in self.data.keys():
            self.data[key][1:-1] = self.data[key][0:-2]
        self.data["LinkQuality"][0], self.data["SNRatio"][0] = self.WirelessStat.WirelessStatRead()


        self.timeData += 1.0
# The drawing method:
        self.setAxisScale(QwtPlot.xBottom, self.timeData[-1], self.timeData[0])
        for key in self.data.keys():
            self.setCurveData(self.keys[key], self.timeData, self.data[key])
# test only:
#        self.WirelessStat.axisScale = 20
#        self.setAxisScale(QwtPlot.yLeft, 0, self.WirelessStat.axisScale)


        self.replot()

    # timerEvent()
    
    def toggleCurve(self, key):
        curve = self.curve(key)
        if curve:
            curve.setEnabled(not curve.enabled())
            self.replot()

    # toggleCurve()

# class WirelessPlot


def main(args):
    qapptitle=["Wireless manager"]
    app = QApplication(qapptitle)
    box = make()
    app.setMainWidget(box)
    app.exec_loop()

# main()

def make():
    box = QVBox()
    plot = WirelessPlot(box)
    plot.setTitle("Continuous read")
    plot.setMargin(5)
    info = QString("          Press the legend to en/disable a curve")
    QLabel(info, box)
    box.resize(600, 400)
    box.show()
    return box

# make()

# Admire!
if __name__ == '__main__':
    main(sys.argv)

# Local Variables: ***
# mode: python ***
# End: ***
